@@ -6,7 +6,7 @@
  *
  * This file is part of ocserv.
  *
- * The GnuTLS is free software; you can redistribute it and/or
+ * ocserv is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
@@ -36,6 +36,7 @@ void *_talloc_size2(void *ctx, size_t size);
 
 #define MAX_IP_STR 46
 
+
 #define PROTOBUF_ALLOCATOR(name, pool) \
 	ProtobufCAllocator name = {.alloc = _talloc_size2, .free = _talloc_free2, .allocator_data = pool}
 
@@ -84,6 +85,7 @@ int recv_msg_data(int fd, uint8_t *cmd, uint8_t *data, size_t data_size, int *re
 
 const char* cmd_request_to_str(unsigned cmd);
 const char* discon_reason_to_str(unsigned reason);
+unsigned int discon_reason_to_log_level(unsigned reason);
 
 ssize_t oc_recvfrom_at(int sockfd, void *buf, size_t len, int flags,
                     struct sockaddr *src_addr, socklen_t *addrlen,
@@ -137,4 +139,8 @@ size_t oc_strlcpy(char *dst, char const *src, size_t siz);
 #define SAFE_ID_SIZE (BASE64_ENCODE_RAW_LENGTH(20)+1)
 char *calc_safe_id(const uint8_t *data, unsigned size, char *output, unsigned output_size);
 
+
+extern int saved_argc;
+extern char **saved_argv;
+
 #endif
