@@ -41,7 +41,7 @@ struct script_wait_st *stmp;
 	stmp->pid = pid;
 
 	ev_child_init(&stmp->ev_child, script_child_watcher_cb, pid, 0);
-	ev_child_start(loop, &stmp->ev_child);
+	ev_child_start(main_loop, &stmp->ev_child);
 
 	list_add(&s->script_list.head, &(stmp->list));
 }
@@ -57,7 +57,7 @@ inline static pid_t remove_from_script_list(main_server_st* s, struct proc_st* p
 	list_for_each_safe(&s->script_list.head, stmp, spos, list) {
 		if (stmp->proc == proc) {
 			list_del(&stmp->list);
-			ev_child_stop(loop, &stmp->ev_child);
+			ev_child_stop(main_loop, &stmp->ev_child);
 			if (stmp->pid > 0) {
 				kill(stmp->pid, SIGTERM);
 				ret = stmp->pid;
