@@ -332,3 +332,22 @@ void *plain_get_brackets_string(void *pool, struct perm_cfg_st *config, const ch
 
 	return additional;
 }
+
+
+void *oidc_get_brackets_string(void * pool, struct perm_cfg_st *config, const char *str)
+{
+	subcfg_val_st vals[MAX_SUBOPTIONS];
+	char * additional = NULL;
+
+	unsigned vals_size, i;
+	
+	vals_size  = expand_brackets_string(pool, str, vals);
+
+	for (i = 0; i < vals_size; i ++)	{
+		if (c_strcasecmp(vals[i].name, "config") == 0) {
+			additional = talloc_strdup(pool, vals[i].value);
+		}
+	}
+	
+	return additional;
+}
