@@ -1244,7 +1244,7 @@ int periodic_check(worker_st * ws, struct timespec *tnow, unsigned dpd)
 	/* we set an alarm at each periodic check to prevent any
 	 * freezes in the worker due to an unexpected block (due to worker
 	 * bug or kernel bug). In that case the worker will be killed due
-	 * the the alarm instead of hanging. */
+	 * the alarm instead of hanging. */
 	terminate_reason = REASON_SERVER_DISCONNECT;
 	alarm(1800);
 
@@ -1709,6 +1709,7 @@ char *replace_vals(worker_st *ws, const char *txt)
 	int ret;
 	str_rep_tab tab[3];
 
+	STR_TAB_INIT(tab, sizeof(tab));
 	STR_TAB_SET(0, "%{U}", ws->username);
 	STR_TAB_SET(1, "%{G}", ws->groupname);
 	STR_TAB_TERM(2);
@@ -2852,4 +2853,4 @@ static int worker_event_loop(struct worker_st * ws)
 	exit_worker_reason(ws, terminate_reason);
 
 	return 1;
-}
\ No newline at end of file
+}
