@@ -39,7 +39,7 @@
 #include <libtasn1.h>
 
 extern const ASN1_ARRAY_TYPE kkdcp_asn1_tab[];
-ASN1_TYPE _kkdcp_pkix1_asn = ASN1_TYPE_EMPTY;
+ASN1_TYPE _kkdcp_pkix1_asn = NULL;
 #endif
 
 extern struct snapshot_t *config_snapshot;
